from collections import defaultdict

class PinError(Exception):
    '''The base class for all Pin errors'''

class ConnectionError(PinError):
    '''Occurs when the library can't reach the Pin API'''

class InvalidResponseError(PinError):
    '''Occurs when the Pin API returns an invalid response'''

class APIError(PinError):
    '''The base class for all errors returned by the API'''

class AuthenticationError(APIError):
    pass

class InvalidResourceError(APIError):
    pass

class InsufficientBalanceError(APIError):
    pass

class ResourceNotFoundError(APIError):
    pass

class CardDeclinedError(APIError):
    pass

class InsufficientFundsError(APIError):
    pass

class ProcessingError(APIError):
    pass

class SuspectedFraudError(APIError):
    pass

class ExpiredCardError(APIError):
    pass

# Map API error codes to their respective error classes
api_errors = defaultdict(lambda: APIError)
api_errors.update(dict(
    unauthenticated = AuthenticationError,
    invalid_resource = InvalidResourceError,
    insufficient_pin_balance = InsufficientBalanceError,
    resource_not_found = ResourceNotFoundError,
    card_declined = CardDeclinedError,
    insufficient_funds = InsufficientFundsError,
    processing_error = ProcessingError,
    suspected_fraud = SuspectedFraudError,
    expired_card = ExpiredCardError
))

